; ---------------------------------------------------------
;	20x4 LCD module SC2004CS*B Driver
; =========================================================
;	(C)2004 t.hara
; ---------------------------------------------------------

	.cpu	300HA

; ---------------------------------------------------------
;	Imports
			.include	"timing.inc"

; ---------------------------------------------------------
;	Exports
			.export		InitializeLcd		; initialize LCD module
			.export		LcdPutc				; send one character to LCD module
			.export		LcdPuts				; send ZERO terminated string to LCD module
			.export		LcdCommand			; send LCD bulk command
			.export		LcdSetFont			; send font data

; ---------------------------------------------------------
;	Register define
LcdDir		.equ		H'FFFFD1			; LCD port data direction register
LcdData		.equ		H'FFFFD3			; LCD port data register

LcdEN		.bequ		5,LcdData			; LCD signal E
LcdRS		.bequ		4,LcdData			; LCD signal RS

; ---------------------------------------------------------
			.section	rom, code

; ---------------------------------------------------------
;	initialize LCD module
;
InitializeLcd:
			push.w		R0					; save
			mov.b		#H'FF, R0L			; set output mode to LCD port
			mov.b		R0L, @LcdDir

			jsr			@Wait4ms
			jsr			@Wait4ms
			jsr			@Wait4ms
			jsr			@Wait4ms

			mov.b		#B'00100011, R0L	; E=1, RS=0, D=03H function set (8bit)
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN
			jsr			@Wait4ms

			mov.b		#B'00100011, R0L	; E=1, RS=0, D=03H function set (8bit)
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN
			jsr			@Wait4ms

			mov.b		#B'00100011, R0L	; E=1, RS=0, D=03H function set (8bit)
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN
			jsr			@Wait4ms

			mov.b		#B'00100010, R0L	; E=1, RS=0, D=02H interface 4bit data length
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN
			jsr			@Wait4ms
			mov.b		#B'00101000, R0L	; function set, DL=4bit, F=5x7dot matrix
			jsr			@LcdCommand
			jsr			@Wait4ms
			mov.b		#B'00000110, R0L	; entry mode set, I/D=increment, S=shift off
			jsr			@LcdCommand
			jsr			@Wait4ms
			mov.b		#B'00001000, R0L	; display on/off, D=display off, C=cursor off, B=blink off
			jsr			@LcdCommand
			jsr			@Wait4ms
			mov.b		#B'00000001, R0L	; display clear
			jsr			@LcdCommand
			jsr			@Wait4ms
			mov.b		#B'00001100, R0L	; display on/off, D=display on, C=cursor off, B=blink off
			jsr			@LcdCommand
			jsr			@Wait4ms
			pop.w		R0					; restore R0
			rts

; ---------------------------------------------------------
;	Wait LCD
WaitLcd:
			push.w		R0
			mov.w		#170, R0
WaitLcdLoop:
			sub.w		#1, R0
			bne			WaitLcdLoop
			pop.w		R0
			rts

; ---------------------------------------------------------
;	Send command to LCD controller
;	input:
;		R0L ... send command byte
;
LcdCommand:
			push.w		R0
;	high 4bit
			mov.b		R0L, R0H			; backup R0L
			shlr.b		R0L					; shift right 4bit
			shlr.b		R0L
			shlr.b		R0L
			shlr.b		R0L
			and.b		#B'00001111, R0L	; E=1, RS=0
			or.b		#B'00100000, R0L
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN				; E=0
			jsr			@WaitLcd
;	low 4bit
			and.b		#B'00001111, R0H	; E=1, RS=0
			or.b		#B'00100000, R0H
			mov.b		R0H, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN				; E=0
			jsr			@WaitLcd
			pop.w		R0
			rts

; ---------------------------------------------------------
;	Send character to LCD controller
;	input:
;		R0L ... send character byte
;
LcdPutc:
			push.w		R0
;	high 4bit
			mov.b		R0L, R0H			; backup R0L
			shlr.b		R0L					; shift right 4bit
			shlr.b		R0L
			shlr.b		R0L
			shlr.b		R0L
			and.b		#B'00001111, R0L	; E=1, RS=1
			or.b		#B'00110000, R0L
			mov.b		R0L, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN				; E=0
			jsr			@WaitLcd
;	low 4bit
			and.b		#B'00001111, R0H	; E=1, RS=1
			or.b		#B'00110000, R0H
			mov.b		R0H, @LcdData
			jsr			@WaitLcd
			bclr		LcdEN				; E=0
			jsr			@WaitLcd
			pop.w		R0
			rts

; ---------------------------------------------------------
;	Send string to LCD controller
;	input:
;		ER1 ... address of string
LcdPuts:
			push.w		R0
			push.l		ER1
LcdPutsLoop:
			mov.b		@ER1, R0L
			beq			LcdPutsLoopExit
			jsr			@LcdPutc
			add.l		#1, ER1
			bra			LcdPutsLoop
LcdPutsLoopExit:
			pop.l		ER1
			pop.w		R0
			rts

; ---------------------------------------------------------
;	Send font data
;	input:
;		R0L ... font number (0...7)
;		ER1 ... font data (+0 ... +7)
;	comment:
;		śALCDJ[\ʒuɏ
;
LcdSetFont:
			push.w		R0
			push.l		ER1

			shll.b		R0L					; convert font number to address
			shll.b		R0L
			shll.b		R0L
			and.b		#B'00111000, R0L
			or.b		#B'01000000, R0L	; command of 'set CG-RAM address'
			jsr			@LcdCommand

			mov.b		#8, R0H				; send font data
LcdSetFontLoop:
			mov.b		@ER1, R0L
			jsr			@LcdPutc
			dec.b		R0H
			bne			LcdSetFontLoop

			mov.b		#B'10000000, R0L	; command of 'set DD-RAM address'
			jsr			@LcdCommand

			pop.l		ER1
			pop.w		R0
			rts

			.end
