; ---------------------------------------------------------
;	BIOS for H8/3048F
; =========================================================
;	(C)2004 t.hara
; ---------------------------------------------------------

	.cpu	300HA

; ---------------------------------------------------------
;	Imports
			.include	"lcddrv.inc"		; LCD driver
			.include	"scidrv.inc"		; SCI driver
			.include	"sram.inc"			; SRAM setup
			.include	"timing.inc"		; software wait
			.include	"trapper.inc"		; interrupt trapper
			.include	"string.inc"		; string utility

; ---------------------------------------------------------
;	Export
			.export		ResetEntry

; ---------------------------------------------------------
			.section	rom, code

; ---------------------------------------------------------
;	Initialize
			.org		H'000100
ResetEntry:
			mov.l		#H'FFFF10, ER7		; set up StackPointer
			jsr			@InitializeSram
			jsr			@InitializeIntTrapper
			jsr			@InitializeLcd
			jsr			@InitializeSci

; ---------------------------------------------------------
;	Bootup message
			mov.b		#LcdClear, R0L
			jsr			@LcdCommand
			jsr			@Wait2ms
			mov.b		#LcdLine0, R0L
			jsr			@LcdCommand
			mov.l		#BootupMessage, ER1
			jsr			@StrPuts

			mov.b		#LcdLine1, R0L
			jsr			@LcdCommand
			mov.l		#CopyrightMessage, ER1
			jsr			@StrPuts

; ---------------------------------------------------------
;	Command interpreter
CommandLine:
			jsr			@ClearPrompt
			mov.b		#LcdLine3, R0L
			jsr			@LcdCommand
			mov.l		#CommandPrompt, ER1
			jsr			@StrPuts

			mov.b		#LcdCursorOn, R0L
			jsr			@LcdCommand
RetryCommandChar:
			jsr			@SciGetc
			bcs			RetryCommandChar

			cmp.b		#"M", R0L
			beq			MotReceiveCommand
			cmp.b		#"m", R0L
			beq			MotReceiveCommand
			cmp.b		#"W", R0L
			beq			MemWriteCommand
			cmp.b		#"w", R0L
			beq			MemWriteCommand
			cmp.b		#"R", R0L
			beq			MemReadCommand
			cmp.b		#"r", R0L
			beq			MemReadCommand
			cmp.b		#"D", R0L
			beq			DumpCommand
			cmp.b		#"d", R0L
			beq			DumpCommand
			bra			RetryCommandChar

; ---------------------------------------------------------
;	Error message
ModeCancel:
			mov.l		#CancelMessage, ER1
			bra			PutMessage
IllegalCharacter:
			mov.l		#IllegalCharacterMessage, ER1
			bra			PutMessage
InvalidLength:
			mov.l		#InvalidLengthMessage, ER1
			bra			PutMessage
NoMatchCheckSum:
			mov.l		#NoMatchCheckSumMessage, ER1
			bra			PutMessage
PutMessage:
			mov.b		#LcdLine1, R0L
			jsr			@LcdCommand
			jsr			@StrPuts
			bra			CommandLine

; ---------------------------------------------------------
;	Dump mode
DumpCommand:
			mov.b		#LcdClear, R0L
			jsr			@LcdCommand
			jsr			@Wait2ms
			mov.b		#LcdLine0, R0L
			jsr			@LcdCommand
			mov.l		#DumpMode, ER1
			jsr			@StrPuts
DumpLoop:
			jsr			@GetAddress				; ER2 <= Address
			bcs			ModeCancel

			mov.l		#CrLf, ER1
			jsr			@SciPuts
			mov.l		#DumpHeader, ER1
			jsr			@SciPuts
			mov.b		#16, R3H				; lines
DumpLineLoop:
			mov.l		ER2, ER1				; dump address
			jsr			@RotEr1_8
			jsr			@RotEr1_8
			mov.b		R1L, R0L
			jsr			@DumpByte
			jsr			@RotEr1_8
			mov.b		R1L, R0L
			jsr			@DumpByte
			jsr			@RotEr1_8
			mov.b		R1L, R0L
			jsr			@DumpByte
			mov.b		#"|", R0L
			jsr			@SciPutc
			mov.b		#16, R3L				; columns
DumpColumnLoop:
			mov.b		@ER2, R0L
			jsr			@DumpByte
			mov.b		#" ", R0L
			jsr			@SciPutc
			inc.l		#1, ER2
			dec.b		R3L
			bne			DumpColumnLoop
			mov.l		#CrLf, ER1
			jsr			@SciPuts
			dec.b		R3H
			bne			DumpLineLoop
			bra			DumpLoop

RotEr1_8:
			rotl.l		ER1						; H'AABBCCDD --> H'BBCCDDAA
			rotl.l		ER1
			rotl.l		ER1
			rotl.l		ER1
			rotl.l		ER1
			rotl.l		ER1
			rotl.l		ER1
			rotl.l		ER1
			rts

; ---------------------------------------------------------
;	Memory write mode
MemWriteCommand:
			mov.b		#LcdClear, R0L
			jsr			@LcdCommand
			jsr			@Wait2ms
			mov.b		#LcdLine0, R0L
			jsr			@LcdCommand
			mov.l		#MemoryWriteMode, ER1
			jsr			@StrPuts
MemWriteLoop:
			jsr			@GetAddress				; ER2 <= Address
			bcs			ModeCancel
			jsr			@GetData				; R3L <= Data
			bcs			ModeCancel
			mov.b		R3L, @ER2
			bra			MemWriteLoop

; ---------------------------------------------------------
;	Memory read mode
MemReadCommand:
			mov.b		#LcdClear, R0L
			jsr			@LcdCommand
			jsr			@Wait2ms
			mov.b		#LcdLine0, R0L
			jsr			@LcdCommand
			mov.l		#MemoryReadMode, ER1
			jsr			@StrPuts
MemReadLoop:
			jsr			@GetAddress				; ER2 <= Address
			bcs			ModeCancel
			mov.b		#LcdLine1, R0L
			jsr			@LcdCommand
			mov.b		@ER2, R0L
			mov.b		R0L, R0H
			shlr.b		R0L
			shlr.b		R0L
			shlr.b		R0L
			shlr.b		R0L
			jsr			@ConvertNumToHex4
			jsr			@LcdPutc
			jsr			@SciPutc
			mov.b		R0H, R0L
			jsr			@ConvertNumToHex4
			jsr			@LcdPutc
			jsr			@SciPutc
			bra			MemReadLoop

; ---------------------------------------------------------
;	Motorola S format receive mode
MotReceiveCommand:
			mov.b		#LcdClear, R0L
			jsr			@LcdCommand
			jsr			@Wait2ms
			mov.b		#LcdLine0, R0L
			jsr			@LcdCommand
			mov.l		#MotorolaSRecieveMode, ER1
			jsr			@StrPuts
			xor.l		ER4, ER4			; Line number
GetRecord:
RetryGetS:
			jsr			@SciGetc
			bcs			RetryGetS

			cmp.b		#10, R0L
			beq			GetRecord			; LF ͖
			cmp.b		#13, R0L
			beq			GetRecord			; CR ͖
			cmp.b		#"S", R0L
			bne			ModeCancel			; 'S' łȂ cancel

; ---------------------------------------------------------
;	'S' command
PutLineNumber:
			mov.b		#LcdLine1, R0L
			jsr			@LcdCommand
			mov.b		#0, R0H				; '0' \邩ۂtO, 0:\Ȃ, 1:\
			mov.l		ER4, ER5
			mov.l		#100000, ER3
			jsr			@PutLineNumberColumn
			mov.l		#10000, ER3
			jsr			@PutLineNumberColumn
			mov.l		#1000, ER3
			jsr			@PutLineNumberColumn
			mov.l		#100, ER3
			jsr			@PutLineNumberColumn
			mov.l		#10, ER3
			jsr			@PutLineNumberColumn
			mov.l		#1, ER3
			jsr			@PutLineNumberColumn
			inc.l		#1, ER4
SetCommand:
			jsr			@SciGetc			; R[hʎ擾
			cmp.b		#"0", R0L
			bcs			ModeCancel			; łȂ cancel
			beq			SetHeadCommand		; '0' Ȃ SetHeadCommand
			cmp.b		#"3"+1, R0L
			bcs			SetDataCommand		; '1'`'3' Ȃ data command
			cmp.b		#"7", R0L
			bcs			ModeCancel			; '4'`'6' Ȃ cancel
			cmp.b		#"9"+1, R0L
			bcc			ModeCancel			; łȂ cancel
			bra			SetRunCommand

; ---------------------------------------------------------
;	'S0' command
SetHeadCommand:
			jsr			@GetByte			; ǂݎ̂
			bcc			SetHeadCommand
			bra			GetRecord			; ̃R[h

; ---------------------------------------------------------
;	'S1' ... 'S3' command
SetDataCommand:
			mov.b		R0L, R2L
			add.b		#0-("1"-2), R2L			; R2 <= 2, 3, 4, 0x31 = "1"

			jsr			@GetByte				; get byte count
			mov.b		R0L, R2H				; R2H is byte count
			mov.b		R0L, R0H				; check-sum
			beq			InvalidLength			; check error
			mov.l		#0, ER3
GetDataAddressLoop:
			jsr			@GetByte
			bcs			IllegalCharacter		; check error
			shll.l		ER3						; ER3 = (ER3 << 8) + R0L
			shll.l		ER3
			shll.l		ER3
			shll.l		ER3
			shll.l		ER3
			shll.l		ER3
			shll.l		ER3
			shll.l		ER3
			mov.b		R0L, R3L
			add.b		R0L, R0H				; make check-sum
			dec.b		R2H
			beq			InvalidLength			; check error
			dec.b		R2L
			bne			GetDataAddressLoop

GetDataFieldLoop:
			jsr			@GetByte
			bcs			IllegalCharacter
			add.b		R0L, R0H				; make check-sum
			dec.b		R2H
			beq			GetDataFieldLoopExit	; check length
			mov.b		R0L, @ER3
			inc.l		#1, ER3					; address increment
			bra			GetDataFieldLoop
GetDataFieldLoopExit:
			or.b		R0H, R0H
			bne			NoMatchCheckSum
			bra			GetRecord				; ̃R[h

; ---------------------------------------------------------
;	'S7' ... 'S9' command
SetRunCommand:
			mov.b		R0L, R2L
			not.b		R2L						; '7','8','9' ---> C8, C7, C6
			add.b		#(2-H'C6) & 255, R2L	; R2 <= 2, 3, 4

			jsr			@GetByte				; get 
			mov.b		R0L, R2H				; R2H is byte count
			beq			InvalidLength			; check error
			mov.l		#0, ER3
GetRunAddressLoop:
			jsr			@GetByte
			bcs			IllegalCharacter		; check error
			shll.l		ER3						; ER3 = (ER3 << 8) + R0L
			shll.l		ER3
			shll.l		ER3
			shll.l		ER3
			shll.l		ER3
			shll.l		ER3
			shll.l		ER3
			shll.l		ER3
			mov.b		R0L, R3L
			dec.b		R2H
			beq			InvalidLength			; check error
			dec.b		R2L
			bne			GetRunAddressLoop

			jsr			@ER3
			bra			ResetEntry				; Av߂Ă\tgZbg

; ---------------------------------------------------------
;	Get address
GetAddress:
			mov.b		#LcdLine3, R0L
			jsr			@LcdCommand
			mov.l		#AddressPrompt, ER1
			jsr			@StrPuts

			mov.b		#LcdLine3+4, R0L
			jsr			@LcdCommand

			xor.l		ER2, ER2			; ER2 <= 0
			xor.b		R0H, R0H			; column
GetAddressLoop:
			jsr			@SciGetc
			bcs			GetAddressLoop
			cmp.b		#8, R0L				; backspace?
			bne			AddressNumber
			or.b		R0H, R0H
			beq			GetAddressLoop		; ͂Ȃ牽Ȃ
			dec.b		R0H					; 1
			mov.b		R0H, R0L
			add.b		#LcdLine3+4, R0L
			jsr			@LcdCommand
			mov.b		#" ", R0L
			jsr			@LcdPutc
			mov.b		R0H, R0L
			add.b		#LcdLine3+4, R0L
			jsr			@LcdCommand
			mov.b		#8, R0L
			jsr			@SciPutc
			shlr.l		ER2
			shlr.l		ER2
			shlr.l		ER2
			shlr.l		ER2
			bra			GetAddressLoop
AddressNumber:
			cmp.b		#13, R0L			; enter?
			beq			GetAddressExit
			cmp.b		#8, R0H				; ł 8͍ς݂Ȃ疳
			bcc			GetAddressLoop
			jsr			@LcdPutc
			jsr			@SciPutc
			jsr			@ConvertHexToNum4
			bcs			GetAddressCancel	; łȂ͂ꂽLZ
			shll.l		ER2
			shll.l		ER2
			shll.l		ER2
			shll.l		ER2
			or.b		R0L, R2L
			inc.b		R0H
			bra			GetAddressLoop
GetAddressExit:
			andc.b		#H'FE, CCR
			rts
GetAddressCancel:
			orc.b		#H'01, CCR
			rts

; ---------------------------------------------------------
;	Get data
GetData:
			mov.b		#LcdLine3, R0L
			jsr			@LcdCommand
			mov.l		#DataPrompt, ER1
			jsr			@StrPuts

			mov.b		#LcdLine3+4, R0L
			jsr			@LcdCommand

			xor.b		R3L, R3L			; R3L <= 0
			xor.b		R0H, R0H			; column
GetDataLoop:
			jsr			@SciGetc
			bcs			GetDataLoop
			cmp.b		#8, R0L				; backspace?
			bne			DataNumber
			or.b		R0H, R0H
			beq			GetDataLoop			; ͂Ȃ牽Ȃ
			dec.b		R0H					; 1
			mov.b		R0H, R0L
			add.b		#LcdLine3+4, R0L
			jsr			@LcdCommand
			mov.b		#" ", R0L
			jsr			@LcdPutc
			mov.b		R0H, R0L
			add.b		#LcdLine3+4, R0L
			jsr			@LcdCommand
			mov.b		#8, R0L
			jsr			@SciPutc
			shlr.b		R3L
			shlr.b		R3L
			shlr.b		R3L
			shlr.b		R3L
			bra			GetDataLoop
DataNumber:
			cmp.b		#13, R0L			; enter?
			beq			GetDataExit
			cmp.b		#2, R0H				; ł 2͍ς݂Ȃ疳
			bcc			GetDataLoop
			jsr			@LcdPutc
			jsr			@SciPutc
			jsr			@ConvertHexToNum4
			bcs			GetDataCancel		; łȂ͂ꂽLZ
			shll.b		R3L
			shll.b		R3L
			shll.b		R3L
			shll.b		R3L
			or.b		R0L, R3L
			inc.b		R0H
			bra			GetDataLoop
GetDataExit:
			andc.b		#H'FE, CCR
			rts
GetDataCancel:
			orc.b		#H'01, CCR
			rts

; ---------------------------------------------------------
;	Get hex number
GetByte:
			push.w		R1
			jsr			@SciGetc
			bcs			GetByteSkip			; check error
			jsr			@ConvertHexToNum4
			bcs			GetByteSkip			; check error
			mov.b		R0L, R1L
			jsr			@SciGetc
			bcs			GetByteSkip			; check error
			jsr			@ConvertHexToNum4
			bcs			GetByteSkip			; check error
			shll.b		R1L
			shll.b		R1L
			shll.b		R1L
			shll.b		R1L
			or.b		R1L, R0L
GetByteSkip:
			pop.w		R1					;  CtO͕ωȂ
			rts

; ---------------------------------------------------------
;	String puts
;	tƃ^[~iɓ(@ER1)o͂
;	ER1 WX^̓e͔j󂳂
StrPuts:
			jsr			@LcdPuts
			jsr			@SciPuts
			mov.l		#CrLf, ER1
			jsr			@SciPuts
			rts

; ---------------------------------------------------------
ClearPrompt:
			mov.b		#LcdLine3, R0L
			jsr			@LcdCommand
			mov.l		#ClearPromptLine, ER1
			jsr			@LcdPuts
			rts

; ---------------------------------------------------------
PutLineNumberColumn:
			xor.b		R0L, R0L
PutLineNumberColumnLoop:
			sub.l		ER3, ER5
			bcs			PutLineNumberColumnLoopExit
			inc.b		R0L
			bra			PutLineNumberColumnLoop
PutLineNumberColumnLoopExit:
			add.l		ER3, ER5				; Ђ߂
			add.b		#"0", R0L				; ɕϊ
			or.b		R0H, R0H				; 0 \ׂH
			bne			PutLineNumberPut
			cmp.b		#"0", R0L				; "0" H
			bne			PutLineNumberPut
			cmp.l		#1, ER3					; 1̈ʂH
			bne			PutLineNumberColumnExit
PutLineNumberPut:
			jsr			@LcdPutc
			mov.b		#1, R0H					; ȍ~͕\
PutLineNumberColumnExit:
			rts

; ---------------------------------------------------------
DumpByte:
			push.l		ER1
			push.w		R0
			mov.b		R0L, R0H
			shlr.b		R0L
			shlr.b		R0L
			shlr.b		R0L
			shlr.b		R0L
			xor.l		ER1, ER1
			mov.b		R0L, R1L
			add.l		#HexCharacter, ER1
			mov.b		@ER1, R0L
			jsr			@SciPutc
			xor.l		ER1, ER1
			mov.b		R0H, R1L
			and.b		#H'0F, R1L
			add.l		#HexCharacter, ER1
			mov.b		@ER1, R0L
			jsr			@SciPutc
			pop.w		R0
			pop.l		ER1
			rts

; ---------------------------------------------------------
;	Datas
CrLf:
			.data.b		H'0D, H'0A, 0
BootupMessage:
			.sdata		"H8 MONITOR v0.10"<0>
CopyrightMessage:
			.sdata		"      (C)2004 t.hara"<0>
CancelMessage:
			.sdata		"Canceled            "<0>
IllegalCharacterMessage:
			.sdata		"E:Illegal Character "<0>
InvalidLengthMessage:
			.sdata		"E:Invalid Length    "<0>
NoMatchCheckSumMessage:
			.sdata		"E:No Match Chk.Sum  "<0>
ClearPromptLine:
			.sdata		"                    "<0>
CommandPrompt:
			.sdata		"CMD>"<0>
AddressPrompt:
			.sdata		"ADR>        "<0>
DataPrompt:
			.sdata		"DAT>        "<0>
MotorolaSRecieveMode:
			.sdata		"MOT RECEIVING.."<0>
MemoryWriteMode:
			.sdata		"MEM WRITE"<0>
MemoryReadMode:
			.sdata		"MEM READ"<0>
DumpMode:
			.sdata		"MEM DUMP"<0>
DumpHeader:
			.sdata		" ADDR |+0 +1 +2 +3 +4 +5 +6 +7 +8 +9 +A +B +C +D +E +F"
			.data.b		H'0D, H'0A
			.sdata		"------+------------------------------------------------"
			.data.b		H'0D, H'0A, 0
HexCharacter:
			.sdata		"0123456789ABCDEF"
			.end
